package com.hp.user.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.common.utils.CollectionUtils;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hp.user.client.entity.AccountVO;
import com.hp.user.client.entity.DataResult;
import com.hp.user.client.entity.ElectricianVO;
import com.hp.user.client.entity.FuzzyQuery;
import com.hp.user.client.entity.ListResult;
import com.hp.user.client.entity.PageQuery;
import com.hp.user.client.entity.PageResult;
import com.hp.user.client.entity.Result;
import com.hp.user.client.service.AccountService;
import com.hp.user.client.service.ElectricianService;
import com.hp.user.service.constants.UserConstants;
import com.hp.user.service.dao.CertificateMapper;
import com.hp.user.service.dao.ElectricianMapper;
import com.hp.user.service.entity.Electrician;
import com.hp.user.service.redis.RedisOperation;
import com.hp.user.service.utils.MapUtil;

import redis.clients.jedis.Jedis;

@Service
public class ElectricianServiceImpl implements ElectricianService{
	
	@Autowired
	private ElectricianMapper electricianMapper;
	
	@Autowired
	private CertificateMapper certificateMapper;
	
	@Autowired
	private AccountService accountService;

	@Override
	@Transactional
	public Result createElectrician(ElectricianVO electricianVO) {
		//TODO 事务还没有加
		Result result = new Result();
		try {
			//创建电工的时候需要绑定一个账号
			String phone = electricianVO.getPhone();
			String name = electricianVO.getName();
			String serviceProviderId = electricianVO.getServiceProviderId();
			if(StringUtils.isBlank(phone)) {
				result.setCode("-1");
				result.setMessage("手机号不能为空！");
				return result;
			}
			if(StringUtils.isBlank(name)) {
				result.setCode("-1");
				result.setMessage("姓名能为空！");
				return result;
			}
			if(StringUtils.isBlank(serviceProviderId)) {
				result.setCode("-1");
				result.setMessage("服务商ID不能为空！");
				return result;
			}
			AccountVO account = new AccountVO();
			account.setCompanyId(serviceProviderId);
			account.setPhone(phone);
			account.setUserName(name);
			DataResult<String> accountResult = accountService.register(account);
			if(accountResult.getCode().equals("0")) {
				Electrician electrician = new Electrician();
				BeanUtils.copyProperties(electricianVO, electrician);
				String electricianId = IdWorker.getIdStr();
				electrician.setElectricianId(electricianId);
				electrician.setUserId(accountResult.getData());
				electricianMapper.insert(electrician);
			}else {
				result.setCode("-1");
				result.setMessage(accountResult.getMessage());
			}
			
		}catch(Exception e) {
			result.setCode("-1");
			result.setMessage("系统内部错误");
			throw e;
		}
		return result;
	}

	@Override
	public PageResult<ElectricianVO> queryElectrician(PageQuery query) {
		PageResult<ElectricianVO> result = new PageResult<ElectricianVO>();
		try {
			Integer page = query.getPage();
			Integer count = query.getSize();
			if(page == null || page == 0) {
				return result;
			}
			QueryWrapper<Electrician> queryWrapper = null;
			List<FuzzyQuery> fuzzyList = query.getFuzzyList();
			if(CollectionUtils.isNotEmpty(fuzzyList)) {
				queryWrapper = new QueryWrapper<Electrician>();
				for(FuzzyQuery<?> fuzzy : fuzzyList) {
					queryWrapper.like(fuzzy.getColumn(), fuzzy.getValue());
				}
			}
			List<Electrician> list = null;
			Long total = null;
			if(page == -1) {
				list = electricianMapper.selectList(queryWrapper);
			}else {
				Page<Electrician> electricianPage = electricianMapper.selectPage(new Page<>(page,count), queryWrapper);
				list = electricianPage.getRecords();
				total = electricianPage.getTotal();
			}
			List<ElectricianVO> newList = new ArrayList<>();
			if(CollectionUtils.isNotEmpty(list)) {
				ElectricianVO electricianVO;
				for(Electrician electrician : list) {
					electricianVO = new ElectricianVO();
					BeanUtils.copyProperties(electrician, electricianVO);
					newList.add(electricianVO);
				}
			}
			result.setTotal(total);
			result.setDataList(newList);
		}catch(Exception e) {
			e.printStackTrace();
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public Result updateElectrician(ElectricianVO electricianVO) {
		Result result = new Result();
		try {
			Electrician electrician = new Electrician(electricianVO.getElectricianId(),electricianVO.getName(),
					electricianVO.getDictionariesId(),electricianVO.getCertificateUrl(),electricianVO.getRemarks(),electricianVO.getStatus());
			electricianMapper.updateById(electrician);
		}catch(Exception e) {
			e.printStackTrace();
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public DataResult<ElectricianVO> queryElectricianById(String electricianId) {
		DataResult<ElectricianVO> result = new DataResult<ElectricianVO>();
		try {
			Electrician electrician = electricianMapper.selectById(electricianId);
			ElectricianVO electricianVO = null;
			if(null != electrician) {
				electricianVO = new ElectricianVO();
				BeanUtils.copyProperties(electrician, electricianVO);
			}
			result.setData(electricianVO);
		}catch(Exception e) {
			e.printStackTrace();
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public Result updateStatusBatch(String[] electricianIds,Integer status) {
		Result result = new Result();
		try {
			UpdateWrapper<Electrician> updateWrapper = new UpdateWrapper<Electrician>();
			updateWrapper.in("electrician_id", electricianIds);
			Electrician electrician = new Electrician();
			electrician.setStatus(status);
			electricianMapper.update(electrician, updateWrapper);
		}catch(Exception e) {
			e.printStackTrace();
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public Result deleteElectricianById(String electricianId) {
		Result result = new Result();
		try {
			UpdateWrapper<Electrician> updateWrapper = new UpdateWrapper<Electrician>();
			updateWrapper.eq("electrician_id", electricianId);
			Electrician electrician = new Electrician();
			electrician.setSysDeleted(1);
			electricianMapper.update(electrician, updateWrapper);
		}catch(Exception e) {
			e.printStackTrace();
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public DataResult<ElectricianVO> queryElectricianByUserId(String userId) {
		DataResult<ElectricianVO> result = new DataResult<ElectricianVO>();
		try {
			QueryWrapper<Electrician> queryWrapper = new QueryWrapper<Electrician>();
			queryWrapper.eq("user_id", userId);
			queryWrapper.eq("sys_deleted", 0);
			Electrician electrician = electricianMapper.selectOne(queryWrapper);
			if(null != electrician) {
				ElectricianVO electricianVO = new ElectricianVO();
				BeanUtils.copyProperties(electrician, electricianVO);
				result.setData(electricianVO);
			}
		}catch(Exception e) {
			e.printStackTrace();
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public Result updateElectricianCid(String electricianId, String cid) {
		Result result = new Result();
		try {
			Electrician electrician = new Electrician();
			electrician.setCid(cid);
			electrician.setElectricianId(electricianId);
			electricianMapper.updateById(electrician);
		}catch(Exception e) {
			e.printStackTrace();
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public ListResult<ElectricianVO> queryElectricianByUserIds(List<String> ids) {
		ListResult<ElectricianVO> result = new ListResult<ElectricianVO>();
		try {
			List<Electrician> list = electricianMapper.selectBatchIds(ids);
			if(!CollectionUtils.isEmpty(list)) {
				ElectricianVO electricianVO;
				List<ElectricianVO> newList = new ArrayList<>();
				for(Electrician electrician : list) {
					electricianVO = new ElectricianVO();
					BeanUtils.copyProperties(electrician, electricianVO);
					newList.add(electricianVO);
				}
				result.setDataList(newList);
			}
		}catch(Exception e) {
			e.printStackTrace();
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public Result updateLocation(String electricianId, String lng, String lat) {
		Result result = new Result();
		Jedis jedis = RedisOperation.getRedis();
		try {
			//将获取到的电工定位信息缓存到redis中
			
			String location = lat+","+lng;
			Integer cityCode = MapUtil.getCityCode(location);
			if(null != cityCode) {
				String key = UserConstants.LOACTION + cityCode;
				jedis.hset(key,electricianId, location+","+System.currentTimeMillis());
			}else {
				result.setCode("-1");
				result.setMessage("定位信息获取失败！");
			}
			
		}catch(Exception e) {
			e.printStackTrace();
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}finally {
			jedis.close();
		}
		return result;
	}
	
	

}
